// condition_variable example from https://cplusplus.com/reference/condition_variable/condition_variable/
#include <iostream>
#include <string>
#include <thread>
#include <mutex>
#include <condition_variable>
std::mutex m;
std::condition_variable cv;
std::string data;
bool ready = false;
bool processed = false;
void worker_thread()
{
    std::unique_lock lk(m);
    cv.wait(lk, []{return ready;}); // Wait until main() sends data, then we own the lock.
    std::cout << "Worker thread is processing data\n";
    data += " after processing";
    processed = true; // Send data back to main()
    std::cout << "Worker thread signals data processing completed\n";
    lk.unlock(); //Manual unlocking is done before notifying
    cv.notify_one();
}
int main()
{
    std::thread worker(worker_thread);
    data = "Example data";
    {
        std::lock_guard lk(m);
        ready = true;
        std::cout << "main() signals data ready\n";
    }
    cv.notify_one();
    {
        std::unique_lock lk(m);
        cv.wait(lk, []{return processed;});// wait for the worker
    }
    std::cout << "Back in main(), data = " << data << '\n';
    worker.join();
}
