//example from: Raynard, James. Learn Multithreading with Modern C++. 
#include <iostream>
#include <future>
#include <queue>
#include <mutex>
#include <condition_variable>
template <class T>
class threadSafeQueue {
	std::mutex m;
	std::queue<T> q;
	std::condition_variable cv;
public:
	threadSafeQueue() = default;
	void push(T value) {
		std::lock_guard<std::mutex> lg(m);
		q.push(value);
		cv.notify_one();
	}

	void pop(T& value) {
		std::unique_lock<std::mutex> lg(m);
		cv.wait(lg, [this] {return !q.empty();});
		value = q.front();
		q.pop();
	}
};
threadSafeQueue<int> myQueue;
void consumer() {
	int data;
	std::cout << "The consumer is running" << std::endl;
	myQueue.pop(data);                                       // Get a value from the queue
	std::cout << "Consumer received: " << data << std::endl;
}
void producer() {
	std::cout << "The producer is running..." << std::endl;
	myQueue.push(10);                                // Push the data onto the queue
	std::cout << "The producer has pushed some data" << std::endl;
}
int main() {
	auto cons = async(std::launch::async, consumer); //starting consumer
	auto prod = async(std::launch::async, producer); //starting producer
	cons.wait();
	prod.wait();
}